@@ -1,9 +1,8 @@
-# $Id: Piece.pm,v 1.13 2002/06/20 06:41:02 matt Exp $
+# $Id: Piece.pm 65 2006-09-05 02:05:11Z matt $
 
 package Time::Piece;
 
 use strict;
-use vars qw($VERSION @ISA @EXPORT %EXPORT_TAGS);
 
 require Exporter;
 require DynaLoader;
@@ -12,25 +11,28 @@ use Carp;
 use Time::Local;
 use UNIVERSAL qw(isa);
 
-@ISA = qw(Exporter DynaLoader);
+our @ISA = qw(Exporter DynaLoader);
 
-@EXPORT = qw(
+our @EXPORT = qw(
     localtime
     gmtime
 );
 
-%EXPORT_TAGS = (
+our %EXPORT_TAGS = (
     ':override' => 'internal',
     );
 
-$VERSION = '1.07';
+our $VERSION = '1.10';
 
 bootstrap Time::Piece $VERSION;
 
 my $DATE_SEP = '-';
 my $TIME_SEP = ':';
 my @MON_LIST = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
+my @FULLMON_LIST = qw(January February March April May June July
+                      August September October November December);
 my @DAY_LIST = qw(Sun Mon Tue Wed Thu Fri Sat);
+my @FULLDAY_LIST = qw(Sunday Monday Tuesday Wednesday Thursday Friday Saturday);
 
 use constant 'c_sec' => 0;
 use constant 'c_min' => 1;
@@ -45,15 +47,19 @@ use constant 'c_epoch' => 9;
 use constant 'c_islocal' => 10;
 
 sub localtime {
-    my $time = shift;
+    unshift @_, __PACKAGE__ unless eval { $_[0]->isa('Time::Piece') };
+    my $class = shift;
+    my $time  = shift;
     $time = time if (!defined $time);
-    _mktime($time, 1);
+    $class->_mktime($time, 1);
 }
 
 sub gmtime {
-    my $time = shift;
+    unshift @_, __PACKAGE__ unless eval { $_[0]->isa('Time::Piece') };
+    my $class = shift;
+    my $time  = shift;
     $time = time if (!defined $time);
-    _mktime($time, 0);
+    $class->_mktime($time, 0);
 }
 
 sub new {
@@ -63,13 +69,13 @@ sub new {
     my $self;
     
     if (defined($time)) {
-        $self = &localtime($time);
+        $self = $class->localtime($time);
     }
     elsif (ref($class) && $class->isa(__PACKAGE__)) {
-        $self = _mktime($class->epoch, $class->[c_islocal]);
+        $self = $class->_mktime($class->epoch, $class->[c_islocal]);
     }
     else {
-        $self = &localtime();
+        $self = $class->localtime();
     }
     
     return bless $self, $class;
@@ -90,16 +96,38 @@ sub parse {
 }
 
 sub _mktime {
-    my ($time, $islocal) = @_;
+    my ($class, $time, $islocal) = @_;
+    $class = eval { (ref $class) && (ref $class)->isa('Time::Piece') }
+           ? ref $class
+           : $class;
     if (ref($time)) {
-	$time->[c_epoch] = undef;
-	return wantarray ? @$time : bless [@$time, $islocal], 'Time::Piece';
+        $time->[c_epoch] = undef;
+        return wantarray ? @$time : bless [@$time, $islocal], $class;
     }
+    _tzset();
     my @time = $islocal ?
             CORE::localtime($time)
                 :
             CORE::gmtime($time);
-    wantarray ? @time : bless [@time, $time, $islocal], 'Time::Piece';
+    wantarray ? @time : bless [@time, $time, $islocal], $class;
+}
+
+my %_special_exports = (
+  localtime => sub { my $c = $_[0]; sub { $c->localtime(@_) } },
+  gmtime    => sub { my $c = $_[0]; sub { $c->gmtime(@_)    } },
+);
+
+sub export {
+  my ($class, $to, @methods) = @_;
+  for my $method (@methods) {
+    if (exists $_special_exports{$method}) {
+      no strict 'refs';
+      no warnings 'redefine';
+      *{$to . "::$method"} = $_special_exports{$method}->($class);
+    } else {
+      $class->SUPER::export($to, $method);
+    }
+  }
 }
 
 sub import {
@@ -162,12 +190,25 @@ sub month {
         return $MON_LIST[$time->[c_mon]];
     }
     else {
-        return $time->strftime('%B');
+        return $time->strftime('%b');
     }
 }
 
 *monname = \&month;
 
+sub fullmonth {
+    my $time = shift;
+    if (@_) {
+        return $_[$time->[c_mon]];
+    }
+    elsif (@FULLMON_LIST) {
+        return $FULLMON_LIST[$time->[c_mon]];
+    }
+    else {
+        return $time->strftime('%B');
+    }
+}
+
 sub year {
     my $time = shift;
     $time->[c_year] + 1900;
@@ -180,7 +221,8 @@ sub _year {
 
 sub yy {
     my $time = shift;
-    $time->[c_year] % 100;
+    my $res = $time->[c_year] % 100;
+    return $res > 9 ? $res : "0$res";
 }
 
 sub wday {
@@ -204,12 +246,25 @@ sub wdayname {
         return $DAY_LIST[$time->[c_wday]];
     }
     else {
-        return $time->strftime('%A');
+        return $time->strftime('%a');
     }
 }
 
 *day = \&wdayname;
 
+sub fullday {
+    my $time = shift;
+    if (@_) {
+        return $_[$time->[c_wday]];
+    }
+    elsif (@FULLDAY_LIST) {
+        return $FULLDAY_LIST[$time->[c_wday]];
+    }
+    else {
+        return $time->strftime('%A');
+    }
+}
+
 sub yday {
     my $time = shift;
     $time->[c_yday];
@@ -227,6 +282,8 @@ sub isdst {
 # Thanks to Tony Olekshy <olekshy@cs.ualberta.ca> for this algorithm
 sub tzoffset {
     my $time = shift;
+    
+    return Time::Seconds->new(0) unless $time->[c_islocal];
 
     my $epoch = $time->epoch;
 
@@ -249,15 +306,15 @@ sub tzoffset {
 sub epoch {
     my $time = shift;
     if (defined($time->[c_epoch])) {
-	return $time->[c_epoch];
+        return $time->[c_epoch];
     }
     else {
-	my $epoch = $time->[c_islocal] ?
-	  timelocal(@{$time}[c_sec .. c_mon], $time->[c_year]+1900)
-	  :
-	  timegm(@{$time}[c_sec .. c_mon], $time->[c_year]+1900);
-	$time->[c_epoch] = $epoch;
-	return $epoch;
+        my $epoch = $time->[c_islocal] ?
+          timelocal(@{$time}[c_sec .. c_mon], $time->[c_year]+1900)
+          :
+          timegm(@{$time}[c_sec .. c_mon], $time->[c_year]+1900);
+        $time->[c_epoch] = $epoch;
+        return $epoch;
     }
 }
 
@@ -302,7 +359,7 @@ sub datetime {
 sub julian_day {
     my $time = shift;
     # Correct for localtime
-    $time = &gmtime( $time->epoch ) if $time->[c_islocal];
+    $time = $time->gmtime( $time->epoch ) if $time->[c_islocal];
 
     # Calculate the Julian day itself
     my $jd = $time->_jd( $time->year, $time->mon, $time->mday,
@@ -393,7 +450,16 @@ sub month_last_day {
 
 sub strftime {
     my $time = shift;
-    my $format = @_ ? shift(@_) : "%a, %d %b %Y %H:%M:%S %Z";
+    my $tzname = $time->[c_islocal] ? '%Z' : 'UTC';
+    my $format = @_ ? shift(@_) : "%a, %d %b %Y %H:%M:%S $tzname";
+    if (!defined $time->[c_wday]) {
+        if ($time->[c_islocal]) {
+            return _strftime($format, CORE::localtime($time->epoch));
+        }
+        else {
+            return _strftime($format, CORE::gmtime($time->epoch));
+        }
+    }
     return _strftime($format, (@$time)[c_sec..c_isdst]);
 }
 
@@ -403,7 +469,7 @@ sub strptime {
     my $format = @_ ? shift(@_) : "%a, %d %b %Y %H:%M:%S %Z";
     my @vals = _strptime($string, $format);
 #    warn(sprintf("got vals: %d-%d-%d %d:%d:%d\n", reverse(@vals)));
-    return scalar _mktime(\@vals, (ref($time) ? $time->[c_islocal] : 0));
+    return scalar $time->_mktime(\@vals, (ref($time) ? $time->[c_islocal] : 0));
 }
 
 sub day_list {
@@ -471,23 +537,29 @@ use overload
 sub subtract {
     my $time = shift;
     my $rhs = shift;
+    if (UNIVERSAL::isa($rhs, 'Time::Seconds')) {
+        $rhs = $rhs->seconds;
+    }
     die "Can't subtract a date from something!" if shift;
     
-    if (ref($rhs) && $rhs->isa('Time::Piece')) {
+    if (UNIVERSAL::isa($rhs, 'Time::Piece')) {
         return Time::Seconds->new($time->epoch - $rhs->epoch);
     }
     else {
         # rhs is seconds.
-        return _mktime(($time->epoch - $rhs), $time->[c_islocal]);
+        return $time->_mktime(($time->epoch - $rhs), $time->[c_islocal]);
     }
 }
 
 sub add {
     my $time = shift;
     my $rhs = shift;
+    if (UNIVERSAL::isa($rhs, 'Time::Seconds')) {
+        $rhs = $rhs->seconds;
+    }
     croak "Invalid rhs of addition: $rhs" if ref($rhs);
 
-    return _mktime(($time->epoch + $rhs), $time->[c_islocal]);
+    return $time->_mktime(($time->epoch + $rhs), $time->[c_islocal]);
 }
 
 use overload
@@ -551,16 +623,18 @@ following methods are available on the object:
     $t->mday                # also available as $t->day_of_month
     $t->mon                 # 1 = January
     $t->_mon                # 0 = January
-    $t->monname             # February
+    $t->monname             # Feb
     $t->month               # same as $t->monname
+    $t->fullmonth           # February
     $t->year                # based at 0 (year 0 AD is, of course 1 BC)
     $t->_year               # year minus 1900
     $t->yy                  # 2 digit year
     $t->wday                # 1 = Sunday
     $t->_wday               # 0 = Sunday
     $t->day_of_week         # 0 = Sunday
-    $t->wdayname            # Tuesday
+    $t->wdayname            # Tue
     $t->day                 # same as wdayname
+    $t->fullday             # Tuesday
     $t->yday                # also available as $t->day_of_year, 0 = Jan 01
     $t->isdst               # also available as $t->daylight_savings
 
